from holidata.holiday import Region
from holidata.utils import day


class VC(Region):
    def __init__(self, country):
        super().__init__("VC", country)

        self.define_holiday() \
            .with_name("Lunes de Fallas") \
            .in_years([2013]) \
            .on(month=3, day=18) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("San Juan") \
            .in_years([2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026]) \
            .on(month=6, day=24) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Día de la Comunitat Valenciana") \
            .except_for([2011, 2016, 2022]) \
            .on(month=10, day=9) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("San José") \
            .in_years([2011, 2012, 2013, 2014, 2015, 2016, 2018, 2019, 2020, 2021, 2022, 2024, 2025, 2026]) \
            .on(month=3, day=19) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Jueves Santo") \
            .in_years([2011, 2016, 2017, 2022, 2023]) \
            .on(day(3).before(country.easter())) \
            .with_flags("RV")

        self.define_holiday() \
            .with_name("Lunes de Pascua") \
            .except_for([2023]) \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        self.define_holiday() \
            .with_name("Lunes siguiente a la Fiesta del Trabajo") \
            .in_years([2011]) \
            .on(month=5, day=2) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Lunes siguiente al Día de la Constitución Española") \
            .in_years([2015]) \
            .on(month=12, day=7) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("San Esteban") \
            .in_years([2016]) \
            .on(month=12, day=26) \
            .with_flags("RF")



from holidata.holiday import Region
from holidata.utils import SmartDayArrow, second, day, first


class TAS(Region):
    def __init__(self, country):
        super().__init__("TAS", country)

        """
        New Year's Day (1 January), unless that day falls on a Saturday or Sunday, in which case the Monday following New Year's Day
        https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096
        """
        self.define_holiday() \
            .with_name("New Year's Day") \
            .on(month=1, day=1) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("New Year's Day (observed)") \
            .on(first("monday").after(month=1, day=1)) \
            .with_flags("V") \
            .on_condition(self.date_is_on_weekend(month=1, day=1))

        """
        Australia Day (26 January), unless that day falls on a Saturday or Sunday, in which case the following Monday
        https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096
        """
        self.define_holiday() \
            .with_name("Australia Day") \
            .on(self.mon_to_fri_on_or_following(month=1, day=26)) \
            .with_flags("V")

        self.define_holiday() \
            .with_name("Royal Hobart Regatta") \
            .in_regions(["TAS"]) \
            .on(second("monday").of("february")) \
            .with_flags("V")

        """
        the second Monday in March, known as Eight Hours Day or Labour Day
        https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .on(second("monday").of("march")) \
            .with_flags("V")

        """
        Good Friday
        https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096
        """
        self.define_holiday() \
            .with_name("Good Friday") \
            .on(day(2).before(country.easter())) \
            .with_flags("RV")

        """
        Easter Monday
        https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096
        """
        self.define_holiday() \
            .with_name("Easter Monday") \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        """
        Anzac Day
        25 April
        https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096
        """
        self.define_holiday() \
            .with_name("Anzac Day") \
            .on(month=4, day=25) \
            .with_flags("F")

        """
        Birthday of the Sovereign
        the second Monday in June
        https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096
        """
        self.define_holiday() \
            .with_name("Queen's Birthday") \
            .until(2022) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        self.define_holiday() \
            .with_name("King's Birthday") \
            .since(2023) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        """
        Christmas Day (25 December); if Christmas Day falls on a Saturday, the Monday following Christmas Day; if Christmas Day falls on a Sunday, the Tuesday following Christmas Day
        https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096
        """
        self.define_holiday() \
            .with_name("Christmas Day") \
            .on(month=12, day=25) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("monday").after(month=12, day=25)) \
            .with_flags("RV") \
            .on_condition(self.date_is_on_weekend(month=12, day=25))

        """
        Boxing Day (26 December), unless that day falls on a Saturday or Sunday, in which case – the Monday following Boxing Day, if that day falls on a Saturday; or the Tuesday following Boxing Day, if that day falls on a Sunday.
        https://www.legislation.tas.gov.au/view/html/inforce/current/act-2000-096
        """
        self.define_holiday() \
            .with_name("Boxing Day") \
            .on(month=12, day=26) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("monday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(self.date_is_saturday(month=12, day=26))

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("tuesday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(self.date_is_sunday_or_monday(month=12, day=26))

    @staticmethod
    def date_is_on_weekend(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["saturday", "sunday"]

        return wrapper

    @staticmethod
    def date_is_saturday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "saturday"

        return wrapper

    @staticmethod
    def date_is_sunday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "sunday"

        return wrapper

    @staticmethod
    def date_is_sunday_or_monday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["sunday", "monday"]

        return wrapper

    @staticmethod
    def monday_on_or_first_monday_following(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).shift_to_weekday("monday", including=True)

        return wrapper

    @staticmethod
    def mon_to_fri_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() in ["saturday", "sunday"]:
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper



from dateutil.easter import EASTER_WESTERN

from holidata.holiday import Country
from holidata.utils import day, first

__all__ = [
    "IT",
]

"""
sources:
- LEGGE 27 maggio   1949, n. 260 https://www.gazzettaufficiale.it/eli/gu/1949/05/31/124/sg
- LEGGE  5 marzo    1977, n.  54 https://www.gazzettaufficiale.it/eli/id/1977/03/07/077U0054/sg 
- LEGGE 20 novembre 2000, n. 336 https://www.gazzettaufficiale.it/eli/id/2000/11/22/000G0390/sg
- DECRETO DEL PRESIDENTE DELLA REPUBBLICA 28 dicembre 1985, n. 792
                                 https://www.gazzettaufficiale.it/eli/id/1985/12/31/085U0792/sg
- LEGGE  8 ottobre  2025, n. 151 https://www.gazzettaufficiale.it/eli/id/2025/10/10/25G00153/sg
"""


class IT(Country):
    id = "IT"
    languages = ["it"]
    default_lang = "it"
    easter_type = EASTER_WESTERN

    def __init__(self):
        super().__init__()

        self.define_holiday() \
            .with_name("Capodanno") \
            .on(month=1, day=1) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Epifania") \
            .on(month=1, day=6) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Festa della liberazione") \
            .on(month=4, day=25) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Festa del lavoro") \
            .on(month=5, day=1) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Festa della repubblica") \
            .on(month=6, day=2) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Santi Apostoli Pietro e Paolo") \
            .on(month=6, day=29) \
            .with_flags("NRF") \
            .annotated_with("Solo per il comune di Roma")

        self.define_holiday() \
            .with_name("Assunzione (ferragosto)") \
            .on(month=8, day=15) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Festa nazionale di San Francesco d'Assisi, patrono d'Italia") \
            .since(2026) \
            .on(month=10, day=4) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Ognissanti") \
            .on(month=11, day=1) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Unità nazionale") \
            .on(first("sunday").of("november")) \
            .with_flags("NF")

        self.define_holiday() \
            .with_name("Immacolata concezione") \
            .on(month=12, day=8) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Natale") \
            .on(month=12, day=25) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("S.to Stefano") \
            .on(month=12, day=26) \
            .with_flags("NRF")

        self.define_holiday() \
            .with_name("Pasqua") \
            .on(self.easter()) \
            .with_flags("NRV")

        self.define_holiday() \
            .with_name("Pasquetta") \
            .on(day(1).after(self.easter())) \
            .with_flags("NRV")



from holidata.holiday import Region
from holidata.utils import day


class NW(Region):
    def __init__(self, country):
        super().__init__("NW", country)

        self.define_holiday() \
            .with_name("Allerheiligen") \
            .on(month=11, day=1) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Fronleichnam") \
            .on(day(60).after(country.easter())) \
            .with_flags("RV")



from holidata.holiday import Region
from holidata.utils import day


class CN(Region):
    def __init__(self, country):
        super().__init__("CN", country)

        self.define_holiday() \
            .with_name("Lunes siguiente a la Epifanía del Señor") \
            .in_years([2013, 2019]) \
            .on(month=1, day=7) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Día de Canarias") \
            .except_for([2021]) \
            .on(month=5, day=30) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Jueves Santo") \
            .on(day(3).before(country.easter())) \
            .with_flags("RV")

        self.define_holiday() \
            .with_name("Lunes de Pascua") \
            .in_years([2022]) \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        self.define_holiday() \
            .with_name("Lunes siguiente a la Fiesta del Trabajo") \
            .in_years([2016]) \
            .on(month=5, day=2) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Lunes siguiente a la Asunción de la Virgen") \
            .in_years([2021]) \
            .on(month=8, day=16) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Lunes siguiente a Todos los Santos") \
            .in_years([2015, 2026]) \
            .on(month=11, day=2) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Lunes siguiente al Día de la Constitución Española") \
            .in_years([2020]) \
            .on(month=12, day=7) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Lunes siguiente a la Natividad del Señor") \
            .in_years([2022]) \
            .on(month=12, day=26) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("San Esteban") \
            .in_years([2011]) \
            .on(month=12, day=26) \
            .with_flags("RF")



from holidata.holiday import Region
from holidata.utils import SmartDayArrow, day, first


class WA(Region):
    def __init__(self, country):
        super().__init__("WA", country)

        """
        New Year's Day
        1st January
        When New Year's Day falls on a Saturday or Sunday the next following Monday is also a public holiday and bank holiday.
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("New Year's Day") \
            .on(month=1, day=1) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("New Year's Day (observed)") \
            .on(first("monday").after(month=1, day=1)) \
            .with_flags("V") \
            .on_condition(self.date_is_on_weekend(month=1, day=1))

        """
        Australia Day
        26th January or, when that day falls on a Saturday or Sunday, the first Monday following the 26th January
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("Australia Day") \
            .on(self.mon_to_fri_on_or_following(month=1, day=26)) \
            .with_flags("V")

        """
        Labour Day
        Monday on or first Monday following the 1st March
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .on(self.monday_on_or_first_monday_following(month=3, day=1)) \
            .with_flags("V")

        """
        Good Friday
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("Good Friday") \
            .on(day(2).before(country.easter())) \
            .with_flags("RV")

        """
        Easter Sunday
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_44571.pdf
        """
        self.define_holiday() \
            .with_name("Easter") \
            .since(2022) \
            .on(country.easter()) \
            .with_flags("RV")

        """
        Easter Monday
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("Easter Monday") \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        """
        Anzac Day
        25th April
        When Anzac Day falls on a Saturday or Sunday the next following Monday is also a public holiday and bank holiday
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("Anzac Day") \
            .on(month=4, day=25) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Anzac Day (supplement)") \
            .on(first("monday").after(month=4, day=25)) \
            .with_flags("F") \
            .on_condition(self.date_is_on_weekend(month=4, day=25))

        """
        Western Australia Day/Foundation Day
        Monday on or first Monday following the 1st June
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("Western Australia Day") \
            .on(self.monday_on_or_first_monday_following(month=6, day=1)) \
            .with_flags("V")

        """
        Birthday of the Sovereign
        day to be appointed for each year by proclamation published in the Government Gazette at least 3 weeks before the day so appointed
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("Queen's Birthday") \
            .until(2022) \
            .on(self.birthday_of_the_sovereign) \
            .with_flags("V")

        self.define_holiday() \
            .with_name("King's Birthday") \
            .since(2023) \
            .on(self.birthday_of_the_sovereign) \
            .with_flags("V")

        """
        National Day of Mourning for Queen Elizabeth II
        2022-09-22
        https://www.legislation.wa.gov.au/legislation/prod/gazettestore.nsf/FileURL/gg2022_138.pdf/$FILE/Gg2022_138.pdf?OpenElement
        """
        self.define_holiday() \
            .with_name("National Day of Mourning for Queen Elizabeth II") \
            .in_years([2022]) \
            .on(month=9, day=22) \
            .with_flags("F")

        """
        Christmas Day
        25th December
        When Christmas Day falls on a Saturday or Sunday the next following Monday is also a public holiday and bank holiday
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("Christmas Day") \
            .on(month=12, day=25) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("monday").after(month=12, day=25)) \
            .with_flags("RV") \
            .on_condition(self.date_is_on_weekend(month=12, day=25))

        """
        Boxing Day
        26th December
        When Boxing Day falls on a Saturday the next following Monday is also a public holiday and bank holiday
        When Boxing Day falls on a Sunday or Monday the next following Tuesday is also a public holiday and bank holiday
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/RedirectURL?OpenAgent&query=mrdoc_19831.pdf
        """
        self.define_holiday() \
            .with_name("Boxing Day") \
            .on(month=12, day=26) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("monday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(self.date_is_saturday(month=12, day=26))

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("tuesday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(self.date_is_sunday_or_monday(month=12, day=26))

    @staticmethod
    def date_is_on_weekend(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["saturday", "sunday"]

        return wrapper

    @staticmethod
    def date_is_saturday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "saturday"

        return wrapper

    @staticmethod
    def date_is_sunday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "sunday"

        return wrapper

    @staticmethod
    def date_is_sunday_or_monday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["sunday", "monday"]

        return wrapper

    @staticmethod
    def monday_on_or_first_monday_following(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).shift_to_weekday("monday", including=True)

        return wrapper

    @staticmethod
    def mon_to_fri_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() in ["saturday", "sunday"]:
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper

    @staticmethod
    def birthday_of_the_sovereign(year):
        """
        Dates proclaimed in the Government Gazettes
        https://www.legislation.wa.gov.au/legislation/statutes.nsf/gazettes.html
        """
        date = {
            2011: {"month": 10, "day": 28},  # 2010 234-6261
            2012: {"month": 10, "day":  1},  # 2008 229-5633
            2013: {"month":  9, "day": 30},  # 2008 229-5633
            2014: {"month":  9, "day": 29},  # 2012  61-1687
            2015: {"month":  9, "day": 28},  # 2012  61-1687
            2016: {"month":  9, "day": 26},  # 2014  74-1595
            2017: {"month":  9, "day": 25},  # 2014  74-1595
            2018: {"month":  9, "day": 24},  # 2016  73-1379
            2019: {"month":  9, "day": 30},  # 2016  73-1379
            2020: {"month":  9, "day": 28},  # 2018  53-1287
            2021: {"month":  9, "day": 27},  # 2018  53-1287
            2022: {"month":  9, "day": 26},  # 2020 151-2919
            2023: {"month":  9, "day": 25},  # 2020 151-2919
            2024: {"month":  9, "day": 23},  # 2022  69-3009
            2025: {"month":  9, "day": 29},  # 2022  69-3009
        }

        return SmartDayArrow(year, **date.get(year)) if date.get(year) is not None else None



from .AT import *
from .AU import *
from .BE import *
from .BR import *
from .CA import *
from .CH import *
from .CO import *
from .CZ import *
from .DE import *
from .DK import *
from .EE import *
from .ES import *
from .FI import *
from .FR import *
from .GB import *
from .GR import *
from .HR import *
from .HU import *
from .IS import *
from .IT import *
from .NL import *
from .NO import *
from .NZ import *
from .PL import *
from .PT import *
from .RU import *
from .SE import *
from .SI import *
from .SK import *
from .TR import *
from .US import *
from .ZA import *



from dateutil.easter import EASTER_WESTERN

from holidata.holiday import Country
from holidata.holidays.AU.ACT import ACT
from holidata.holidays.AU.NSW import NSW
from holidata.holidays.AU.NT import NT
from holidata.holidays.AU.QLD import QLD
from holidata.holidays.AU.SA import SA
from holidata.holidays.AU.TAS import TAS
from holidata.holidays.AU.VIC import VIC
from holidata.holidays.AU.WA import WA

__all__ = [
    "AU"
]


class AU(Country):
    id = "AU"
    languages = ["en"]
    default_lang = "en"
    regions = ["ACT", "NSW", "NT", "QLD", "SA", "TAS", "VIC", "WA"]
    easter_type = EASTER_WESTERN

    def __init__(self):
        super().__init__()

        self.regions = [
            ACT(self),
            NSW(self),
            NT(self),
            QLD(self),
            SA(self),
            TAS(self),
            VIC(self),
            WA(self),
        ]



from holidata.holiday import Region
from holidata.utils import day


class PV(Region):
    def __init__(self, country):
        super().__init__("PV", country)

        self.define_holiday() \
            .with_name("V Centenario Vuelta al Mundo") \
            .in_years([2022]) \
            .on(month=9, day=6) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("80º aniversario del primer Gobierno Vasco") \
            .in_years([2016]) \
            .on(month=10, day=7) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Día del País Vasco-Euskadiko Eguna") \
            .in_years([2011, 2012, 2013, 2014]) \
            .on(month=10, day=25) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("San José") \
            .in_years([2015, 2019, 2020, 2021, 2026]) \
            .on(month=3, day=19) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Jueves Santo") \
            .on(day(3).before(country.easter())) \
            .with_flags("RV")

        self.define_holiday() \
            .with_name("Lunes de Pascua") \
            .except_for([2012]) \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        self.define_holiday() \
            .with_name("Santiago Apóstol") \
            .in_years([2011, 2013, 2015, 2016, 2017, 2019, 2020, 2022]) \
            .on(month=7, day=25) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Santiago Apóstol / Día Nacional de Galicia") \
            .in_years([2023, 2024, 2025, 2026]) \
            .on(month=7, day=25) \
            .with_flags("RF")




from holidata.holiday import Region
from holidata.utils import SmartDayArrow, day, first, second


class NT(Region):
    def __init__(self, country):
        super().__init__("NT", country)

        """
        New Year's Day
        1 January, and, if that day falls on a Saturday or Sunday, the following Monday
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("New Year's Day") \
            .on(month=1, day=1) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("New Year's Day (observed)") \
            .on(first("monday").after(month=1, day=1)) \
            .with_flags("V") \
            .on_condition(self.date_is_on_weekend(month=1, day=1))

        """
        Australia Day
        26 January, or, if that day falls on a Saturday or a Sunday, the following Monday
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("Australia Day") \
            .on(self.mon_to_fri_on_or_following(month=1, day=26)) \
            .with_flags("V")

        """
        Good Friday
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("Good Friday") \
            .on(day(2).before(country.easter())) \
            .with_flags("RV")

        """
        The Saturday following Good Friday
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("Easter Saturday") \
            .on(day(1).before(country.easter())) \
            .with_flags("RV")

        """
        The Sunday following Good Friday
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("Easter") \
            .on(country.easter()) \
            .with_flags("RV")

        """
        The Monday following Good Friday
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("Easter Monday") \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        """
        Anzac Day
        25 April, or, if that day falls on a Sunday, the following Monday
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("Anzac Day") \
            .on(self.mon_to_sat_on_or_following(month=4, day=25)) \
            .with_flags("V")

        """
        May Day
        The first Monday in May
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("May Day") \
            .on(first("monday").of("may")) \
            .with_flags("V")

        """
        Birthday of the Sovereign
        The second Monday in June
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("King's Birthday") \
            .since(2023) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        """
        Picnic Day
        The first Monday in August
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("Picnic Day") \
            .on(first("monday").of("august")) \
            .with_flags("V")

        """
        24 December (Christmas Eve) from 7.00 pm to midnight
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        """
        Christmas Day
        25 December, and, if that day falls on a Saturday or Sunday, the following Monday
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("Christmas Day") \
            .on(month=12, day=25) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("monday").after(month=12, day=25)) \
            .with_flags("RV") \
            .on_condition(self.date_is_on_weekend(month=12, day=25))

        """
        Boxing Day
        26 December and, if that day falls on a Saturday, the following Monday, or, if that day falls on a Sunday or Monday, the following Tuesday
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """
        self.define_holiday() \
            .with_name("Boxing Day") \
            .on(month=12, day=26) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("monday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(self.date_is_saturday(month=12, day=26))

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("tuesday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(self.date_is_sunday_or_monday(month=12, day=26))

        """
        New Year's Eve
        31 December from 7.00 pm to midnight
        https://legislation.nt.gov.au/Legislation/PUBLIC-HOLIDAYS-ACT-1981
        """

    @staticmethod
    def date_is_on_weekend(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["saturday", "sunday"]

        return wrapper

    @staticmethod
    def date_is_saturday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "saturday"

        return wrapper

    @staticmethod
    def date_is_sunday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "sunday"

        return wrapper

    @staticmethod
    def date_is_sunday_or_monday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["sunday", "monday"]

        return wrapper

    @staticmethod
    def mon_to_fri_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() in ["saturday", "sunday"]:
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper

    @staticmethod
    def mon_to_sat_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() == "sunday":
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper



from holidata.holiday import Region
from holidata.utils import SmartDayArrow, day, second, first


class NSW(Region):
    def __init__(self, country):
        super().__init__("NSW", country)

        """
        New Year's Day
        Public holiday on 1 January.
        When 1 January is a Saturday or Sunday, there is to be an additional public holiday on the following Monday.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("New Year's Day") \
            .on(month=1, day=1) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("New Year's Day (observed)") \
            .on(first("monday").after(month=1, day=1)) \
            .with_flags("V") \
            .on_condition(NSW.date_is_on_weekend(month=1, day=1))

        """
        Australia Day
        Public holiday on 26 January.
        When 26 January is a Saturday or Sunday, there is to be no public holiday on that day and instead the following Monday is to be a public holiday.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("Australia Day") \
            .on(self.mon_to_fri_on_or_following(month=1, day=26)) \
            .with_flags("V")

        """
        Good Friday
        Public holiday on the Friday publicly observed as Good Friday.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("Good Friday") \
            .on(day(2).before(country.easter())) \
            .with_flags("RV")

        """
        Easter Saturday
        Public holiday on the day after Good Friday.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("Easter Saturday") \
            .on(day(1).before(country.easter())) \
            .with_flags("RV")

        """
        Easter Sunday
        Public holiday on the Sunday following Good Friday.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("Easter") \
            .on(country.easter()) \
            .with_flags("RV")

        """
        Easter Monday
        Public holiday on the Monday following Good Friday.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("Easter Monday") \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        """
        Anzac Day
        Public holiday on 25 April.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("Anzac Day") \
            .on(month=4, day=25) \
            .with_flags("F")

        """
        Queen's Birthday
        Public holiday on the second Monday in June.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        """
        self.define_holiday() \
            .with_name("Queen's Birthday") \
            .until(2022) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        """
        King's Birthday
        Public holiday on the second Monday in June.
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("King's Birthday") \
            .since(2023) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        """
        Labour Day
        Public holiday on the first Monday in October.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .on(first("monday").of("october")) \
            .with_flags("V")

        """
        Christmas Day
        Public holiday on 25 December.
        When 25 December is a Saturday, there is to be an additional public holiday on the following Monday.
        When 25 December is a Sunday, there is to be an additional public holiday on the following Tuesday.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("Christmas Day") \
            .on(month=12, day=25) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("monday").after(month=12, day=25)) \
            .with_flags("RV") \
            .on_condition(NSW.date_is_saturday(month=12, day=25))

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("tuesday").after(month=12, day=25)) \
            .with_flags("RV") \
            .on_condition(NSW.date_is_sunday(month=12, day=25))

        """
        Boxing Day
        Public holiday on 26 December.
        When 26 December is a Saturday, there is to be an additional public holiday on the following Monday.
        When 26 December is a Sunday, there is to be an additional public holiday on the following Tuesday.
        https://legislation.nsw.gov.au/view/html/inforce/2011-12-31/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2012-07-06/act-2010-115
        https://legislation.nsw.gov.au/view/html/inforce/2023-01-13/act-2010-115
        """
        self.define_holiday() \
            .with_name("Boxing Day") \
            .on(month=12, day=26) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("monday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(NSW.date_is_saturday(month=12, day=26))

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("tuesday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(NSW.date_is_sunday(month=12, day=26))

    @staticmethod
    def date_is_on_weekend(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["saturday", "sunday"]

        return wrapper

    @staticmethod
    def date_is_saturday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "saturday"

        return wrapper

    @staticmethod
    def date_is_sunday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "sunday"

        return wrapper

    @staticmethod
    def mon_to_fri_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() in ["saturday", "sunday"]:
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper



from holidata.holiday import Region
from holidata.utils import day


class TH(Region):
    """
    According to Thüringer Feier- und Gedenktagsgesetz (ThürFGtG) § 2 Abs. 1, § 2 Abs. 2 und 3 Nr. 1
    (https://landesrecht.thueringen.de/perma?d=jlr-FeiertGTHV5P2)
    the holidays are
     - Neujahrstag (New Years Day), *
     - Karfreitag (Good Friday), *
     - Ostermontag (Easter Monday), *
     - 1. Mai (May 1st), *
     - Tag Christi Himmelfahrt (Ascension), *
     - Pfingstmontag (Pentecost Monday), *
     - 20. September als Weltkindertag (Universal Children's Day),
     - 3. Oktober als Tag der Deutschen Einheit (Day of German Unity), *
     - Reformationstag (Reformation Day),
     - erster Weihnachtsfeiertag (Christmas Day), *
     - zweiter Weihnachtsfeiertag (Boxing Day), *
     - Fronleichnam (Corpus Christi, in communities with a catholic majority)

     Holidays marked with '*' are already covered by the definitions in __init__.py
    """
    def __init__(self, country):
        super().__init__("TH", country)

        self.define_holiday() \
            .with_name("Weltkindertag") \
            .since(2019) \
            .on(month=9, day=20) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Reformationstag") \
            .until(2016) \
            .on(month=10, day=31) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Reformationstag") \
            .since(2018) \
            .on(month=10, day=31) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Fronleichnam") \
            .on(day(60).after(country.easter())) \
            .with_flags("RV") \
            .annotated_with("In Gemeinden mit überwiegend katholischer Bevölkerung")



from holidata.holiday import Region
from holidata.utils import SmartDayArrow, first, second, day


class ACT(Region):
    def __init__(self, country):
        super().__init__("ACT", country)

        """
        New Year's Day
        1 January, and, if that day falls on a Saturday or Sunday, the following Monday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("New Year's Day") \
            .on(month=1, day=1) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("New Year's Day (observed)") \
            .on(first("monday").after(month=1, day=1)) \
            .with_flags("V") \
            .on_condition(ACT.date_is_on_weekend(month=1, day=1))

        """
        Australia Day
        26 January, or, if that day falls on a Saturday or Sunday, the following Monday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Australia Day") \
            .on(self.mon_to_fri_on_or_following(month=1, day=26)) \
            .with_flags("V")

        """
        Canberra Day
        the 2nd Monday in March;
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Canberra Day") \
            .on(second("monday").of("march")) \
            .with_flags("V")

        """
        Good Friday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Good Friday") \
            .on(day(2).before(country.easter())) \
            .with_flags("RV")

        """
        the Saturday following Good Friday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Easter Saturday") \
            .on(day(1).before(country.easter())) \
            .with_flags("RV")

        """
        Easter Sunday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Easter") \
            .on(country.easter()) \
            .with_flags("RV")

        """
        the Monday following Good Friday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Easter Monday") \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        """
        Anzac Day
        25 April, or, if that day falls on a Sunday, the following Monday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Anzac Day") \
            .on(self.mon_to_sat_on_or_following(month=4, day=25)) \
            .with_flags("V")

        """
        Reconciliation Day
        27 May, or, if that day is not a Monday, the following Monday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Reconciliation Day") \
            .on(first("monday").after(month=5, day=26, including=True)) \
            .with_flags("V")

        """
        Birthday of the Sovereign
        the 2nd Monday in June
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Queen's Birthday") \
            .until(2022) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        self.define_holiday() \
            .with_name("King's Birthday") \
            .since(2023) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        """
        Bank Holiday
        the 1st Monday in August
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Bank Holiday") \
            .on(first("monday").of("august")) \
            .with_flags("V")

        """
        Labour Day
        the 1st Monday in October
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .on(first("monday").of("october")) \
            .with_flags("V")

        """
        Christmas Day
        25 December, and, if that day falls on a Saturday, the following Monday, or if that day falls on a Sunday, the following Tuesday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Christmas Day") \
            .on(month=12, day=25) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("monday").after(month=12, day=25)) \
            .with_flags("RV") \
            .on_condition(ACT.date_is_saturday(month=12, day=25))

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("tuesday").after(month=12, day=25)) \
            .with_flags("RV") \
            .on_condition(ACT.date_is_sunday(month=12, day=25))

        """
        Boxing Day
        26 December and, if that day falls on a Saturday, the following Monday, or, if that day falls on a Sunday, the following Tuesday
        https://www.legislation.act.gov.au/View/a/1958-19/current/html/1958-19.html
        """
        self.define_holiday() \
            .with_name("Boxing Day") \
            .on(month=12, day=26) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("monday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(ACT.date_is_saturday(month=12, day=26))

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("tuesday").after(month=12, day=26)) \
            .with_flags("RV") \
            .on_condition(ACT.date_is_sunday(month=12, day=26))

    @staticmethod
    def date_is_on_weekend(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["saturday", "sunday"]

        return wrapper

    @staticmethod
    def date_is_saturday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "saturday"

        return wrapper

    @staticmethod
    def date_is_sunday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "sunday"

        return wrapper

    @staticmethod
    def mon_to_fri_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() in ["saturday", "sunday"]:
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper

    @staticmethod
    def mon_to_sat_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() == "sunday":
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper



from holidata.holiday import Region
from holidata.utils import SmartDayArrow, day, second, first


class SA(Region):
    def __init__(self, country):
        super().__init__("SA", country)

        """
        1 January.
        (a) when a day mentioned in Part 2 of Schedule 2 falls on a Saturday, the
        following Monday will be a public holiday instead of that day and that day
        and the following Monday will be bank holidays; and
        (b) when a day mentioned in Part 2 of Schedule 2 falls on a Sunday, that day and
        the following Monday will be public holidays and bank holidays.
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("New Year's Day") \
            .on(month=1, day=1) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("New Year's Day (observed)") \
            .on(first("monday").after(month=1, day=1)) \
            .with_flags("V") \
            .on_condition(self.date_is_on_weekend(month=1, day=1))

        """
        Australia Day
        26 January.
        (a) when a day mentioned in Part 2 of Schedule 2 falls on a Saturday, the
        following Monday will be a public holiday instead of that day and that day
        and the following Monday will be bank holidays; and
        (b) when a day mentioned in Part 2 of Schedule 2 falls on a Sunday, that day and
        the following Monday will be public holidays and bank holidays.
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("Australia Day") \
            .on(self.mon_to_fri_on_or_following(month=1, day=26)) \
            .with_flags("V")

        """
        Good Friday.
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("Good Friday") \
            .on(day(2).before(country.easter())) \
            .with_flags("RV")

        """
        The day after Good Friday.
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("Easter Saturday") \
            .on(day(1).before(country.easter())) \
            .with_flags("RV")

        """
        Easter Sunday
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("Easter") \
            .on(country.easter()) \
            .with_flags("RV")

        """
        Easter Monday.
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("Easter Monday") \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        """
        Anzac Day
        In addition to the days mentioned in Schedule 2, 25 April will be a public holiday and
        bank holiday but when that day falls on a Sunday, that day and the following Monday
        will be public holidays and bank holidays.
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("Anzac Day") \
            .on(month=4, day=25) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("Anzac Day (observed)") \
            .on(first("monday").after(month=4, day=25)) \
            .with_flags("V") \
            .on_condition(self.date_is_sunday(month=4, day=25))

        """
        Adelaide Cup Day
        Proclaimed as a replacement for the third Monday in May.
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("Adelaide Cup Day") \
            .on(self.date_as_declared_by_the_holidays_adelaide_cup_proclamation) \
            .with_flags("V")

        """
        Birthday of the Sovereign
        The second Monday in June.
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("King's Birthday") \
            .since(2023) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        """
        National Day of Mourning for Queen Elizabeth II
        2022-09-22, as proclaimed by the Holidays (National Day of Mourning for Queen Elizabeth II) Proclamation 2022 (Gazette 16.9.2022 p 6013)
        https://www.legislation.sa.gov.au/__legislation/lz/v/p/2022/holidays%20(national%20day%20of%20mourning%20for%20queen%20elizabeth%20ii)%20proclamation%202022_16.9.2022%20p%206013/16.9.2022%20p%206013.un.pdf
        """
        self.define_holiday() \
            .with_name("National Day of Mourning for Queen Elizabeth II") \
            .in_years([2022]) \
            .on(month=9, day=22) \
            .with_flags("F")

        """
        Labour Day
        The first Monday in October.
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .on(first("monday").of("october")) \
            .with_flags("V")

        """
        Christmas Day.
        (a) when a day mentioned in Part 2 of Schedule 2 falls on a Saturday, the
        following Monday will be a public holiday instead of that day and that day
        and the following Monday will be bank holidays; and
        (b) when a day mentioned in Part 2 of Schedule 2 falls on a Sunday, that day and
        the following Monday will be public holidays and bank holidays. 
        https://www.legislation.sa.gov.au/__legislation/lz/c/a/holidays%20act%201910/current/1910.1010.auth.pdf
        """
        self.define_holiday() \
            .with_name("Christmas Day") \
            .on(month=12, day=25) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("monday").after(month=12, day=25)) \
            .with_flags("RV") \
            .on_condition(self.date_is_on_weekend(month=12, day=25))

    @staticmethod
    def date_is_on_weekend(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["saturday", "sunday"]

        return wrapper

    @staticmethod
    def date_is_saturday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "saturday"

        return wrapper

    @staticmethod
    def date_is_sunday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "sunday"

        return wrapper

    @staticmethod
    def mon_to_fri_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() in ["saturday", "sunday"]:
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper

    @staticmethod
    def date_as_declared_by_the_holidays_adelaide_cup_proclamation(year):
        substitution_date = {
            2011: {"month": 3, "day": 14},  # Holidays (Adelaide Cup) Proclamation 2010
            2012: {"month": 3, "day": 12},  # Holidays (Adelaide Cup) Proclamation 2011
            2013: {"month": 3, "day": 11},  # Holidays (Adelaide Cup) Proclamation 2012
            2014: {"month": 3, "day": 10},
            2015: {"month": 3, "day":  9},
            2016: {"month": 3, "day": 14},
            2017: {"month": 3, "day": 13},
            2018: {"month": 3, "day": 12},
            2019: {"month": 3, "day": 11},
            2020: {"month": 3, "day":  9},
            2021: {"month": 3, "day":  8},
            2022: {"month": 3, "day": 14},
            2023: {"month": 3, "day": 13},
            2024: {"month": 3, "day": 11},  # subject to Proclamation
            2025: {"month": 3, "day": 10},  # subject to Proclamation
        }

        return SmartDayArrow(year, **substitution_date.get(year)) if substitution_date.get(year) is not None else None



from holidata.holiday import Region


class ST(Region):
    def __init__(self, country):
        super().__init__("ST", country)

        self.define_holiday() \
            .with_name("Heilige drei Könige") \
            .on(month=1, day=6) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Reformationstag") \
            .until(2016) \
            .on(month=10, day=31) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Reformationstag") \
            .since(2018) \
            .on(month=10, day=31) \
            .with_flags("RF")



from holidata.holiday import Region
from holidata.utils import SmartDayArrow, day, first, second


class QLD(Region):
    def __init__(self, country):
        super().__init__("QLD", country)

        """
        New Year's Day
        Public holiday on 1 January.
        When 1 January is a Saturday or Sunday, there is to be an additional public holiday on the following Monday.
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        https://www.legislation.qld.gov.au/view/html/2011-12-06/act-1983-018
        """
        self.define_holiday() \
            .with_name("New Year's Day") \
            .on(month=1, day=1) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("New Year's Day (observed)") \
            .since(2012) \
            .on(first("monday").after(month=1, day=1)) \
            .with_flags("V") \
            .on_condition(self.date_is_on_weekend(month=1, day=1))

        """
        Australia Day
        Public holiday on 26 January.
        When 26 January is a Saturday or Sunday, there is to be no public holiday on that day and instead the following Monday is to be a public holiday.
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        https://www.legislation.qld.gov.au/view/html/2011-12-06/act-1983-018
        """
        self.define_holiday() \
            .with_name("Australia Day") \
            .on(self.mon_to_fri_on_or_following(month=1, day=26)) \
            .with_flags("V")

        """
        Good Friday
        Public holiday on the Friday publicly observed as Good Friday.
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        """
        self.define_holiday() \
            .with_name("Good Friday") \
            .on(day(2).before(country.easter())) \
            .with_flags("RV")

        """
        Easter Saturday
        Public holiday on the day after Good Friday.
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        """
        self.define_holiday() \
            .with_name("Easter Saturday") \
            .on(day(1).before(country.easter())) \
            .with_flags("RV")

        """
        Easter Sunday
        Public holiday on the Sunday following Good Friday.
        https://www.legislation.qld.gov.au/view/html/2017-03-01/act-1983-018
        """
        self.define_holiday() \
            .with_name("Easter") \
            .since(2017) \
            .on(country.easter()) \
            .with_flags("RV")

        """
        Easter Monday
        Public holiday on the Monday following Good Friday.
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        """
        self.define_holiday() \
            .with_name("Easter Monday") \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        """
        Anzac Day
        Public holiday on 25 April.
        25 April (Anzac Day) A public holiday is to be observed on—
        (a) 25 April; or
        (b) if 25 April is a Sunday—the following Monday.
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        https://www.legislation.qld.gov.au/view/html/2011-12-06/act-1983-018
        """
        self.define_holiday() \
            .with_name("Anzac Day") \
            .on(self.mon_to_sat_on_or_following(month=4, day=25)) \
            .with_flags("V")

        """
        Labour Day
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .until(2011) \
            .on(month=5, day=1) \
            .with_flags("F")

        """
        A public holiday is to be observed on—
        (a) 1 May; or
        (b) if 1 May is a day other than a Monday—the following Monday.
        https://www.legislation.qld.gov.au/view/html/2011-12-06/act-1983-018
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .since(2012) \
            .until(2012) \
            .on(self.monday_on_or_following(month=5, day=1)) \
            .with_flags("V")

        """
        A public holiday is to be observed on the first Monday in October.
        https://www.legislation.qld.gov.au/view/html/2012-11-08/act-1983-018
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .since(2013) \
            .until(2015) \
            .on(first("monday").of("october")) \
            .with_flags("V")

        """
        A public holiday is to be observed on the first Monday in May.
        https://www.legislation.qld.gov.au/view/html/2015-10-22/act-1983-018
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .since(2016) \
            .on(first("monday").of("may")) \
            .with_flags("V")

        """
        Birthday of the Sovereign
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        """
        self.define_holiday() \
            .with_name("Queen's Birthday") \
            .until(2011) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        """
        https://www.legislation.qld.gov.au/view/html/2011-12-06/act-1983-018
        """
        self.define_holiday() \
            .with_name("Queen's Birthday") \
            .since(2012) \
            .until(2012) \
            .on(first("monday").of("october")) \
            .with_flags("V")

        """
        https://www.legislation.qld.gov.au/view/html/2012-11-08/act-1983-018
        """
        self.define_holiday() \
            .with_name("Queen's Birthday") \
            .since(2013) \
            .until(2015) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        """
        https://www.legislation.qld.gov.au/view/html/2015-10-22/act-1983-018
        """
        self.define_holiday() \
            .with_name("Queen's Birthday") \
            .since(2016) \
            .until(2021) \
            .on(first("monday").of("october")) \
            .with_flags("V")

        self.define_holiday() \
            .with_name("King's Birthday") \
            .since(2022) \
            .on(first("monday").of("october")) \
            .with_flags("V")

        """
        Queen's Diamond Jubilee
        https://www.legislation.qld.gov.au/view/html/2011-12-06/act-1983-018
        """
        self.define_holiday() \
            .with_name("Queen's Diamond Jubilee") \
            .in_years([2012]) \
            .on(month=6, day=11) \
            .with_flags("F")

        """
        National Day of Mourning for Her Majesty The Queen
        https://www.legislation.qld.gov.au/view/html/2022-09-15/act-1983-018
        """
        self.define_holiday() \
            .with_name("National Day of Mourning for Her Majesty The Queen") \
            .in_years([2022]) \
            .on(month=9, day=22) \
            .with_flags("F")

        """
        Christmas Day
        Public holiday on 25 December.
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        https://www.legislation.qld.gov.au/view/html/2011-12-06/act-1983-018
        """
        self.define_holiday() \
            .with_name("Christmas Day") \
            .on(month=12, day=25) \
            .with_flags("RF")

        """
        27 December
        A public holiday is to be observed on 27 December only if 25 December is a Saturday or Sunday.
        """
        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(month=12, day=27) \
            .with_flags("RF") \
            .on_condition(self.date_is_on_weekend(month=12, day=25))

        """
        Boxing Day
        Public holiday on 26 December.
        https://www.legislation.qld.gov.au/view/html/2010-12-10/act-1983-018
        https://www.legislation.qld.gov.au/view/html/2011-12-06/act-1983-018
        """
        self.define_holiday() \
            .with_name("Boxing Day") \
            .on(month=12, day=26) \
            .with_flags("RF")

        """
        28 December 
        A public holiday is to be observed on 28 December only if 26 December is a Saturday or Sunday.
        """
        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(month=12, day=28) \
            .with_flags("RF") \
            .on_condition(self.date_is_on_weekend(month=12, day=26))

    @staticmethod
    def date_is_on_weekend(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["saturday", "sunday"]

        return wrapper

    @staticmethod
    def date_is_saturday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "saturday"

        return wrapper

    @staticmethod
    def date_is_sunday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "sunday"

        return wrapper

    @staticmethod
    def monday_on_or_following(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).shift_to_weekday("monday", including=True)

        return wrapper

    @staticmethod
    def mon_to_fri_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() in ["saturday", "sunday"]:
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper

    @staticmethod
    def mon_to_sat_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() == "sunday":
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper



from holidata.holiday import Region
from holidata.utils import SmartDayArrow, second, day, first, last


class VIC(Region):
    def __init__(self, country):
        super().__init__("VIC", country)

        """
        New Year's Day
        1 January, and the Monday after 1 January (New Year's Day) when New Year's Day is a Saturday or Sunday
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("New Year's Day") \
            .on(month=1, day=1) \
            .with_flags("F")

        self.define_holiday() \
            .with_name("New Year's Day (observed)") \
            .on(first("monday").after(month=1, day=1)) \
            .with_flags("V") \
            .on_condition(self.date_is_on_weekend(month=1, day=1))

        """
        Australia Day
        26 January, or the Monday after Australia Day when Australia Day is a Saturday or Sunday
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf        
        """
        self.define_holiday() \
            .with_name("Australia Day") \
            .on(VIC.mon_to_fri_on_or_following(month=1, day=26)) \
            .with_flags("V")

        """
        Labour Day
        the second Monday in March
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Labour Day") \
            .on(second("monday").of("march")) \
            .with_flags("V")

        """
        Good Friday
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Good Friday") \
            .on(day(2).before(country.easter())) \
            .with_flags("RV")

        """
        the Saturday before Easter Sunday
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Easter Saturday") \
            .on(day(1).before(country.easter())) \
            .with_flags("RV")

        """
        Easter Sunday
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Easter") \
            .since(2019) \
            .on(country.easter()) \
            .with_flags("RV")

        """
        Easter Monday
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Easter Monday") \
            .on(day(1).after(country.easter())) \
            .with_flags("RV")

        """
        Anzac Day
        25 April
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Anzac Day") \
            .on(month=4, day=25) \
            .with_flags("F")

        """
        Birthday of the Sovereign
        the second Monday in June
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Queen's Birthday") \
            .until(2022) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        self.define_holiday() \
            .with_name("King's Birthday") \
            .since(2023) \
            .on(second("monday").of("june")) \
            .with_flags("V")

        """
        Grand Final Eve
        the Friday before the Australian Football League Grand Final
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Grand Final Eve") \
            .since(2019) \
            .on(VIC.friday_before_afl_grand_final) \
            .with_flags("V")

        """
        Melbourne Cup Day
        the first Tuesday in November
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Melbourne Cup Day") \
            .on(first("tuesday").of("november")) \
            .with_flags("V")

        """
        25 December (Christmas Day) or the Monday after Christmas Day when Christmas Day is a Saturday or the Tuesday after Christmas Day when Christmas Day is a Sunday
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Christmas Day") \
            .on(month=12, day=25) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("monday").after(month=12, day=25)) \
            .with_flags("RF") \
            .on_condition(self.date_is_saturday(month=12, day=25))

        self.define_holiday() \
            .with_name("Christmas Day (observed)") \
            .on(first("tuesday").after(month=12, day=25)) \
            .with_flags("RF") \
            .on_condition(self.date_is_sunday(month=12, day=25))

        """
        26 December (Boxing Day)
        the Monday after 26 December (Boxing Day) when Boxing Day is a Saturday or the Tuesday after Boxing Day when Boxing Day is a Sunday
        2009: https://content.legislation.vic.gov.au/sites/default/files/ccbe13db-9862-3b4f-b16c-8f9a552d2dbc_93-119a024.pdf
        2011: https://content.legislation.vic.gov.au/sites/default/files/68a646e7-a547-39aa-adac-b2d5e4bfa756_93-119aa025%20authorised.pdf
        2019: https://content.legislation.vic.gov.au/sites/default/files/ad24ad2c-06f2-3ae3-b0ce-1fcd9b5f61a0_93-119aa026%20authorised.pdf
        """
        self.define_holiday() \
            .with_name("Boxing Day") \
            .on(month=12, day=26) \
            .with_flags("RF")

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("monday").after(month=12, day=26)) \
            .with_flags("RF") \
            .on_condition(self.date_is_saturday(month=12, day=26))

        self.define_holiday() \
            .with_name("Boxing Day (observed)") \
            .on(first("tuesday").after(month=12, day=26)) \
            .with_flags("RF") \
            .on_condition(self.date_is_sunday(month=12, day=26))

    @staticmethod
    def date_is_on_weekend(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() in ["saturday", "sunday"]

        return wrapper

    @staticmethod
    def date_is_saturday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "saturday"

        return wrapper

    @staticmethod
    def date_is_sunday(month, day):
        def wrapper(year):
            return SmartDayArrow(year, month, day).weekday() == "sunday"

        return wrapper

    @staticmethod
    def friday_before_afl_grand_final(year):
        """
        Usually, the Friday before last Saturday in September
        """
        exception = {
                2011: {"month":  9, "day": 31},  # MCG was occupied by the International Cricket Council (ICC)
                2015: {"month": 10, "day":  2},  # Due to scheduling of the 2015 Rugby World Cup
                2016: {"month":  9, "day": 31},
                2020: {"month": 10, "day": 23},  # Adaption due to the COVID-19 pandemic
        }

        if year in exception:
            return SmartDayArrow(year, **exception[year])

        return first("friday").before(last("saturday").of("september"))(year)

    @staticmethod
    def mon_to_fri_on_or_following(month, day):
        def wrapper(year):
            date = SmartDayArrow(year, month, day)

            if date.weekday() in ["saturday", "sunday"]:
                date.shift_to_weekday("monday", including=True)

            return date

        return wrapper
